/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#include "operator/operator.pmh"

using namespace fe;

void BrushTestOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<I32>("Refinement")=0;

	catalog<Real>("Normal Scale")=1.0;
	catalog<Real>("Normal Scale","high")=1.0;

	catalog<Real>("Outline Offset")=0.01;
	catalog<Real>("Outline Offset","high")=0.1;

	catalog< sp<Component> >("Brush");
	catalog<String>("Brush","implementation")="DrawI";

	catalog< sp<Component> >("Input Surface");
}

void BrushTestOp::handle(Record& a_rSignal)
{
	SpatialVector line[4];

	sp<DrawI> spDrawBrush;
	accessBrush(spDrawBrush,a_rSignal,e_quiet);

	if(spDrawBrush.isValid())
	{
		if(m_spInput.isValid())
		{
			const SpatialVector& rRayOrigin=rayOrigin(a_rSignal);
			const SpatialVector& rRayDirection=rayDirection(a_rSignal);

			const Real maxDistance=100.0;
			sp<SurfaceI::ImpactI> spImpact=m_spInput->rayImpact(
					rRayOrigin,rRayDirection,maxDistance);
			if(spImpact.isValid())
			{
				const Real normalScale=catalog<Real>("Normal Scale");
				const Real outlineOffset=catalog<Real>("Outline Offset");

				const SpatialVector intersection=spImpact->intersection();
				const SpatialVector normal=spImpact->normal();

				m_event.bind(windowEvent(a_rSignal));
				WindowEvent::MouseButtons buttons=m_event.mouseButtons();
				BWORD pressed=(buttons&WindowEvent::e_mbLeft);

				line[0]=intersection;
				line[1]=intersection+normalScale*normal*(pressed? 3.0: 1.0);

				spDrawBrush->drawLines(line,NULL,2,DrawI::e_strip,false,NULL);

				sp<SurfaceTriangles::Impact> spTriImpact=spImpact;
				if(spTriImpact.isValid())
				{
					line[0]=spTriImpact->vertex0()+
							outlineOffset*spTriImpact->normal0();
					line[1]=spTriImpact->vertex1()+
							outlineOffset*spTriImpact->normal1();
					line[2]=spTriImpact->vertex2()+
							outlineOffset*spTriImpact->normal2();
					line[3]=line[0];
					spDrawBrush->drawLines(line,NULL,4,DrawI::e_strip,
							false,NULL);

					line[0]=spTriImpact->vertex0();
					line[1]=line[0]+normalScale*spTriImpact->normal0();
					spDrawBrush->drawLines(line,NULL,2,DrawI::e_strip,
							false,NULL);

					line[0]=spTriImpact->vertex1();
					line[1]=line[0]+normalScale*spTriImpact->normal1();
					spDrawBrush->drawLines(line,NULL,2,DrawI::e_strip,
							false,NULL);

					line[0]=spTriImpact->vertex2();
					line[1]=line[0]+normalScale*spTriImpact->normal2();
					spDrawBrush->drawLines(line,NULL,2,DrawI::e_strip,
							false,NULL);
				}
			}
		}
		else
		{
			spDrawBrush->drawLines(line,NULL,3,DrawI::e_strip,false,NULL);
		}

		m_keepInput=TRUE;

		return;
	}

	if(!m_keepInput)
	{
		if(!access(m_spInput,"Input Surface")) return;

		m_spInput->setRefinement(catalog<I32>("Refinement"));
	}
	m_keepInput=FALSE;

	sp<DrawI> spDrawGuide;
	if(!accessGuide(spDrawGuide,a_rSignal)) return;

	sp<DrawableI> spDrawable=m_spInput;
	if(spDrawable.isValid())
	{
		spDrawable->draw(spDrawGuide,NULL);
	}
}
